#include <stdint.h>

#include "mylib1_source2.h"

const char* mylib1_source2_func1 () {
  
  return "Mylib1__mylib1_source2_func1";
}

const char* mylib1_source2_func2 () {
  
  return "Mylib1__mylib1_source2_func2";
}
