package SPVM::Packer::Specifier;



1;

=head1 Name

SPVM::Packer::Specifier - A Specifier for a pack/unpack Template.

=head1 Description

Packer::Specifier class in L<SPVM> represents has a specifier for a pack/unpack template.

This class is a private class for L<Packer|SPVM::Packer> class.

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

