=head1 Name

SPVM::Document::Modules - SPVM Standard Modules

=head1 Description

This document lists standard modules in the SPVM language. Standard modules are the modules contained in the distribution of the SPVM language.

=head1 SPVM Standard Modules

=over 2

=item * L<SPVM::Address|SPVM::Address>

=item * L<SPVM::Array|SPVM::Array>

=item * L<SPVM::Bool|SPVM::Bool>

=item * L<SPVM::Byte|SPVM::Byte>

=item * L<SPVM::ByteList|SPVM::ByteList>

=item * L<SPVM::Callback|SPVM::Callback>

=item * L<SPVM::Callback::Grep|SPVM::Callback::Grep>

=item * L<SPVM::Callback::Grep::Byte|SPVM::Callback::Grep::Byte>

=item * L<SPVM::Callback::Grep::Double|SPVM::Callback::Grep::Double>

=item * L<SPVM::Callback::Grep::Float|SPVM::Callback::Grep::Float>

=item * L<SPVM::Callback::Grep::Int|SPVM::Callback::Grep::Int>

=item * L<SPVM::Callback::Grep::Long|SPVM::Callback::Grep::Long>

=item * L<SPVM::Callback::Grep::Short|SPVM::Callback::Grep::Short>

=item * L<SPVM::Callback::Map|SPVM::Callback::Map>

=item * L<SPVM::Callback::MapExpand|SPVM::Callback::MapExpand>

=item * L<SPVM::Cloneable|SPVM::Cloneable>

=item * L<SPVM::Cloner|SPVM::Cloner>

=item * L<SPVM::CommandInfo|SPVM::CommandInfo>

=item * L<SPVM::Comparable|SPVM::Comparable>

=item * L<SPVM::Comparator|SPVM::Comparator>

=item * L<SPVM::Comparator::Double|SPVM::Comparator::Double>

=item * L<SPVM::Comparator::Float|SPVM::Comparator::Float>

=item * L<SPVM::Comparator::Int|SPVM::Comparator::Int>

=item * L<SPVM::Comparator::Long|SPVM::Comparator::Long>

=item * L<SPVM::Comparator::String|SPVM::Comparator::String>

=item * L<SPVM::Complex_2d|SPVM::Complex_2d>

=item * L<SPVM::Complex_2f|SPVM::Complex_2f>

=item * L<SPVM::Countable|SPVM::Countable>

=item * L<SPVM::Double|SPVM::Double>

=item * L<SPVM::DoubleList|SPVM::DoubleList>

=item * L<SPVM::EqualityCheckable|SPVM::EqualityCheckable>

=item * L<SPVM::EqualityChecker|SPVM::EqualityChecker>

=item * L<SPVM::EqualityChecker::Address|SPVM::EqualityChecker::Address>

=item * L<SPVM::Error|SPVM::Error>

=item * L<SPVM::Error::Compile|SPVM::Error::Compile>

=item * L<SPVM::Error::FieldNotSpecified|SPVM::Error::FieldNotSpecified>

=item * L<SPVM::Error::MethodCallNotPermitted|SPVM::Error::MethodCallNotPermitted>

=item * L<SPVM::Error::NotSupported|SPVM::Error::NotSupported>

=item * L<SPVM::Error::System|SPVM::Error::System>

=item * L<SPVM::Error::Unicode::InvalidUTF8|SPVM::Error::Unicode::InvalidUTF8>

=item * L<SPVM::Float|SPVM::Float>

=item * L<SPVM::FloatList|SPVM::FloatList>

=item * L<SPVM::Fn|SPVM::Fn>

=item * L<SPVM::Format|SPVM::Format>

=item * L<SPVM::Hash|SPVM::Hash>

=item * L<SPVM::Hash::Entry|SPVM::Hash::Entry>

=item * L<SPVM::Immutable::ByteList|SPVM::Immutable::ByteList>

=item * L<SPVM::Immutable::DoubleList|SPVM::Immutable::DoubleList>

=item * L<SPVM::Immutable::FloatList|SPVM::Immutable::FloatList>

=item * L<SPVM::Immutable::IntList|SPVM::Immutable::IntList>

=item * L<SPVM::Immutable::LongList|SPVM::Immutable::LongList>

=item * L<SPVM::Immutable::ShortList|SPVM::Immutable::ShortList>

=item * L<SPVM::Immutable::StringList|SPVM::Immutable::StringList>

=item * L<SPVM::Int|SPVM::Int>

=item * L<SPVM::IntList|SPVM::IntList>

=item * L<SPVM::List|SPVM::List>

=item * L<SPVM::Long|SPVM::Long>

=item * L<SPVM::LongList|SPVM::LongList>

=item * L<SPVM::Native|SPVM::Native>

=item * L<SPVM::Native::API|SPVM::Native::API>

=item * L<SPVM::Native::Arg|SPVM::Native::Arg>

=item * L<SPVM::Native::BasicType|SPVM::Native::BasicType>

=item * L<SPVM::Native::ClassFile|SPVM::Native::ClassFile>

=item * L<SPVM::Native::ClassVar|SPVM::Native::ClassVar>

=item * L<SPVM::Native::Compiler|SPVM::Native::Compiler>

=item * L<SPVM::Native::Constant|SPVM::Native::Constant>

=item * L<SPVM::Native::Env|SPVM::Native::Env>

=item * L<SPVM::Native::Field|SPVM::Native::Field>

=item * L<SPVM::Native::Method|SPVM::Native::Method>

=item * L<SPVM::Native::MethodCall|SPVM::Native::MethodCall>

=item * L<SPVM::Native::Runtime|SPVM::Native::Runtime>

=item * L<SPVM::Native::Stack|SPVM::Native::Stack>

=item * L<SPVM::Packer|SPVM::Packer>

=item * L<SPVM::Packer::Specifier|SPVM::Packer::Specifier>

=item * L<SPVM::Point|SPVM::Point>

=item * L<SPVM::Point3D|SPVM::Point3D>

=item * L<SPVM::QList|SPVM::QList>

=item * L<SPVM::Scope::Guard|SPVM::Scope::Guard>

=item * L<SPVM::Short|SPVM::Short>

=item * L<SPVM::ShortList|SPVM::ShortList>

=item * L<SPVM::Sort|SPVM::Sort>

=item * L<SPVM::SPVM|SPVM::SPVM>

=item * L<SPVM::Stringable|SPVM::Stringable>

=item * L<SPVM::StringBuffer|SPVM::StringBuffer>

=item * L<SPVM::Stringer|SPVM::Stringer>

=item * L<SPVM::StringList|SPVM::StringList>

=item * L<SPVM::Sync::Mutex|SPVM::Sync::Mutex>

=back

=head1 SPVM CPAN Modules

=over 2

=item * L<SPVM::Compress::Raw::Zlib>

=item * L<SPVM::Digest::MD5>

=item * L<SPVM::Digest::SHA>

=item * L<SPVM::Errno>

=item * L<SPVM::File::Basename>

=item * L<SPVM::File::Copy>

=item * L<SPVM::File::Find>

=item * L<SPVM::File::Glob>

=item * L<SPVM::File::Path>

=item * L<SPVM::File::Spec>

=item * L<SPVM::File::Temp>

=item * L<SPVM::FindBin>

=item * L<SPVM::Getopt::Long>

=item * L<SPVM::Go>

=item * L<SPVM::HTTP::Tiny>

=item * L<SPVM::IO>

=item * L<SPVM::IO::Socket::SSL>

=item * L<SPVM::JSON>

=item * L<SPVM::Math>

=item * L<SPVM::MIME::Base64>

=item * L<SPVM::Mojolicious>

=item * L<SPVM::Mozilla::CA>

=item * L<SPVM::Net::DNS::Native>

=item * L<SPVM::Net::SSLeay>

=item * L<SPVM::R>

=item * L<SPVM::Regex>

=item * L<SPVM::Resource::Eigen>

=item * L<SPVM::Resource::Libpng>

=item * L<SPVM::Resource::RE2>

=item * L<SPVM::Resource::SocketUtil>

=item * L<SPVM::Resource::Utf8proc>

=item * L<SPVM::Resource::Zlib>

=item * L<SPVM::Sys>

=item * L<SPVM::Thread>

=item * L<SPVM::Time::Local>

=item * L<SPVM::Time::Piece>

=item * L<SPVM::Unicode::Normalize>

=back

See also L<SPVM CPAN Modules - SPVM Wiki|https://github.com/yuki-kimoto/SPVM/wiki/CPAN-Modules>

=head1 See Also

=over 2

=item * L<SPVM::Document>

=back

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
